'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

exports.loadingReferTree = loadingReferTree;
exports.loadingReferGrid = loadingReferGrid;
exports.getRefinfo = getRefinfo;
exports.getLazyTreeData = getLazyTreeData;
exports.getTreeData = getTreeData;
exports.getGridData = getGridData;
exports.doReload = doReload;
exports.clearPreDatas = clearPreDatas;
exports.saveInnerFilterObject = saveInnerFilterObject;
exports.delInnerFilterObject = delInnerFilterObject;

var _ReferConstants = require('../constants/ReferConstants');

var ajax = require('../../../utils/ajax');
var TreeUtils = require("../../../utils/TreeUtils");
var DataUtil = require('../../../utils/DataUtil');
var querystring = require('query-string');
var _ = require('lodash');
var Cookies = require('js-cookie');

/*按照id获取参照配置表数据*/
function getRefinfoByIdSuccess(key, data) {
    return {
        type: _ReferConstants.GET_REFINFO_BYCODE_SUCCESS,
        key: key,
        data: data
    };
}

function getRefinfoByIdFail(flag) {
    return {
        type: _ReferConstants.GET_REFINFO_BYCODE_ERROR,
        flag: flag
    };
}

/*获取树数据*/
var getTreeDataSuccess = function getTreeDataSuccess(data) {
    return function (dispatch) {
        dispatch({
            type: _ReferConstants.GET_TREEDATA_REF_SUCCESS,
            data: data
        });
        return Promise.resolve();
    };
};

function getTreeDataFail(flag) {
    return {
        type: _ReferConstants.GET_TREEDATA_REF_ERROR,
        flag: flag
    };
}

/*获取表数据*/
function getGridDataSuccess(data) {
    return {
        type: _ReferConstants.GET_GRIDDATA_REF_SUCCESS,
        data: data
    };
}

function getGridDataFail(flag) {
    return {
        type: _ReferConstants.GET_GRIDDATA_REF_ERROR,
        flag: flag
    };
}

/*获取树表数据: 表*/
function getTreeGridDataSuccess(data) {
    return {
        type: _ReferConstants.GET_TREEGRIDDATA_REF_SUCCESS,
        data: data
    };
}

function getTreeGridDataFail(flag) {
    return {
        type: _ReferConstants.GET_TREEGRIDDATA_REF_ERROR,
        flag: flag
    };
}

/*获取树表数据: 树数据+表数据*/
function getTreeAndGridDataSuccess(treeData, treeGridData) {
    return {
        type: _ReferConstants.GET_TREE_AND_GRID_REF_SUCCESS,
        treeData: treeData,
        treeGridData: treeGridData
    };
}

// /*获取常用*/
// function getOftenSuccess(data) {
//     return {
//         type: GET_REFER_OFTEN_SUCCESS,
//         data
//     };
// }
//
// function getOftenError(data) {
//     return {
//         type: GET_REFER_OFTEN_ERROR,
//         data
//     };
// }
//
// /*保存常用*/
// function saveOftenSuccess(data, ifShowMsg) {
//     return {
//         type: SAVE_REFER_OFTEN_SUCCESS,
//         ifShowMsg: ifShowMsg,
//         data
//     };
// }
// function saveOftenError(errorMsg) {
//     return {
//         type: SAVE_REFER_OFTEN_ERROR,
//         errorMsg: errorMsg
//     };
// }
//
// /*删除常用*/
// function deleteOftenSuccess(data) {
//     return {
//         type: DELETE_REFER_OFTEN_SUCCESS,
//         data
//     };
// }
// function deleteOftenError(data) {
//     return {
//         type: DELETE_REFER_OFTEN_ERROR,
//         errorMsg: errorMsg
//     };
// }
function loadingReferTree(loading) {
    return {
        type: _ReferConstants.LOADING_REFER_TREE,
        loadingReferTree: loading
    };
}

function loadingReferGrid(loading) {
    return {
        type: _ReferConstants.LOADING_REFER_GRID,
        loadingReferGrid: loading
    };
}

function getRefinfo(url, key, querydata) {
    return function (dispatch) {
        ajax.request({
            method: 'GET',
            url: url,
            query: querydata,
            type: 'json',
            accept: '',
            success: function success(data) {
                console.log('getRefinfo:', data, typeof data === 'undefined' ? 'undefined' : _typeof(data));
                return dispatch(getRefinfoByIdSuccess(key, data));
            },
            error: function error(data) {
                return dispatch(getRefinfoByIdFail(true));
            }
        });
    };
}

/**
 * 异步树加载
 * @param treeData
 * @param treeNode
 */
function getLazyTreeData(options) {
    var dataurl = options.dataurl,
        dispatch = options.dispatch,
        curKey = options.curKey,
        treeData = options.treeData,
        resolve = options.resolve,
        queryData = options.queryData;

    queryData = _.assign({}, queryData, { pid: curKey });
    ajax.getJSON(dataurl, queryData, function (result) {
        var loop = function loop(data) {
            _.forEach(data, function (item) {
                if (curKey == item.id) {
                    item.children = result;
                } else {
                    if (item.children) {
                        loop(item.children);
                    }
                }
            });
        };
        loop(treeData);
        dispatch(getTreeDataSuccess(treeData)).then(function () {
            resolve();
        });
    }, function (error) {
        resolve();
    });
}

// queryType === REFER_TYPE_TREEGRID情况下， dataurl、queryGrid生效
function getTreeData(dataurl, querydata, queryType, queryGrid) {
    return function (dispatch) {
        dispatch(loadingReferTree(true));
        var header = {
            method: queryGrid && queryGrid.queryMethod ? queryGrid.queryMethod : 'GET',
            url: dataurl,
            send: querydata,
            query: querydata,
            type: 'json',
            accept: 'text'
        };
        if (header.method.toLowerCase() == 'get') {
            delete header.send;
        } else if (header.method.toLowerCase() == "post") {
            delete header.query;
        }
        // ncc的参照数据服务特殊处理
        var technicalType = queryGrid && queryGrid.technicalType ? queryGrid.technicalType : undefined;
        if (technicalType === "ncc") {
            header.noToken = true;
            header.cookies = true;
            var params = header.send || header.query || {};
            header.send = convertNccTreeRefParams(params, queryGrid.treeCondition);
        }
        ajax.request(_.assign({}, header, {
            success: function success(data) {
                //如果data=""空字符串 treeData={}
                var treeData = !!data ? JSON.parse(data) : {};
                // ncc数据格式特殊处理
                if (technicalType === "ncc") {
                    // 组装根节点
                    if (queryGrid && queryGrid.rootPk && queryGrid.rootName) {
                        treeData = treeData.data && treeData.data.length ? [{
                            id: queryGrid.rootPk,
                            code: queryGrid.rootPk,
                            name: queryGrid.rootName,
                            isLeaf: false,
                            selectable: false,
                            children: treeData.data
                        }] : [];
                    } else {
                        treeData = treeData.data;
                    }
                }
                if (queryType === _ReferConstants.REFER_TYPE_TREEGRID) {
                    // 树表参照, 查询树后 默认根据第一个根的第一个子节点查询grid数据
                    var querydata = queryGrid.querydata;
                    if (treeData && treeData[0] && queryGrid && queryGrid.dataurl) {
                        // 第二次发起请求，失败的话，dispatch(getGridDataSuccess(data))
                        var nodeKey = null;
                        if (queryGrid.nodeKey && TreeUtils.findById(treeData, queryGrid.nodeKey)) {
                            nodeKey = queryGrid.nodeKey;
                        } else {
                            if (treeData[0].children && treeData[0].children[0]) {
                                nodeKey = treeData[0].children[0].id;
                            } else {
                                nodeKey = treeData[0].id;
                            }
                        }
                        if (querydata) {
                            var fullCondition = querydata.relyCondition;
                            // queryGrid.relyfield 是必传的参数，如果没有，说明程序错误。所以此处不加非空判断
                            if (fullCondition) {
                                fullCondition += "&" + queryGrid.relyfield + "=" + nodeKey;
                            } else {
                                fullCondition = queryGrid.relyfield + "=" + nodeKey;
                            }
                            querydata.relyCondition = fullCondition;
                        }
                        dispatch(loadingReferGrid(true));
                        var options = {
                            method: queryGrid.queryMethod || 'GET',
                            url: queryGrid.dataurl,
                            send: querydata,
                            query: querydata,
                            type: 'json',
                            success: function success(gridData) {
                                // ncc数据结果集做转换
                                if (technicalType === "ncc") {
                                    var result = gridData.data;
                                    gridData.data = convertNccGridRefResult(result);
                                }
                                dispatch(loadingReferGrid(false));
                                return dispatch(getTreeAndGridDataSuccess(treeData, gridData));
                            },
                            error: function error(gridData) {
                                dispatch(loadingReferGrid(false));
                                // return dispatch(getTreeGridDataFail(true));
                                return dispatch(getTreeDataSuccess(treeData));
                            }
                        };
                        if (options.method.toLowerCase() == 'get') {
                            delete options.send;
                        } else if (options.method.toLowerCase() == "post") {
                            delete options.query;
                        }
                        // ncc的参照数据服务特殊处理
                        if (technicalType === "ncc") {
                            options.noToken = true;
                            options.cookies = true;
                            var _params = options.send || options.query || {};
                            options.send = convertNccGridRefParams(_params, queryGrid.queryCondition);
                        }
                        ajax.request(options);
                    }
                }
                dispatch(loadingReferTree(false));
                return dispatch(getTreeDataSuccess(treeData));
            },
            error: function error(data) {
                dispatch(loadingReferTree(false));
                return dispatch(getTreeDataFail(true));
            }
        }));
    };
}

function getGridData(dataurl, querydata, queryType, options) {
    var _ref = options || {},
        queryMethod = _ref.queryMethod,
        searchObject = _ref.searchObject,
        technicalType = _ref.technicalType,
        queryCondition = _ref.queryCondition;

    return function (dispatch) {
        dispatch(loadingReferGrid(true));
        // 将内嵌行筛选的条件传递后台
        if (_.isObject(searchObject) && !_.isEmpty(searchObject)) {
            // 删除searchObject里面value为空的项
            for (var key in searchObject) {
                if (!searchObject['' + key]) {
                    delete searchObject['' + key];
                }
            }
            if (!_.isEmpty(searchObject)) {
                querydata.searchObject = JSON.stringify(searchObject);
            }
        }
        var queryOptions = {
            method: queryMethod || 'GET',
            url: dataurl,
            send: querydata,
            query: querydata,
            type: 'json',
            success: function success(data) {
                // ncc数据结果集做转换
                if (technicalType === "ncc") {
                    var result = data.data;
                    data.data = convertNccGridRefResult(result);
                }
                dispatch(loadingReferGrid(false));
                if (queryType === _ReferConstants.REFER_TYPE_TREEGRID) {
                    return dispatch(getTreeGridDataSuccess(data));
                } else {
                    return dispatch(getGridDataSuccess(data));
                }
            },
            error: function error(data) {
                dispatch(loadingReferGrid(false));
                if (queryType === _ReferConstants.REFER_TYPE_TREEGRID) {
                    return dispatch(getTreeGridDataFail(true));
                } else {
                    return dispatch(getGridDataFail(true));
                }
            }
        };
        if (queryOptions.method.toLowerCase() == 'get') {
            delete queryOptions.send;
        } else if (queryOptions.method.toLowerCase() == "post") {
            delete queryOptions.query;
        }
        // ncc的参照数据服务特殊处理
        if (technicalType === "ncc") {
            queryOptions.noToken = true;
            queryOptions.cookies = true;
            var params = queryOptions.send || queryOptions.query || {};
            queryOptions.send = convertNccGridRefParams(params, queryCondition);
        }
        ajax.request(queryOptions);
    };
}

/**
 * 针对ncc的列表型参照,对数据请求的参数做转换
 * @param params 参照组件传递的参数
 * @param queryCondition 过滤条件
 * @author xg
 */
function convertNccGridRefParams(params, queryCondition) {
    var _ref2 = params || {},
        pageNumber = _ref2.pageNumber,
        pageSize = _ref2.pageSize,
        condition = _ref2.condition,
        searchText = _ref2.searchText,
        relyCondition = _ref2.relyCondition;

    if (!condition) {
        condition = {};
    } else if (!_.isPlainObject(condition)) {
        condition = JSON.parse(condition);
    }
    if (!queryCondition) {
        queryCondition = {};
    } else if (!_.isPlainObject(queryCondition)) {
        queryCondition = JSON.parse(queryCondition);
    }
    var busiParamJson = {
        pageInfo: { pageSize: pageSize, pageIndex: pageNumber - 1 },
        pid: "",
        keyword: searchText,
        defineItems: [],
        queryCondition: _.assign({}, queryCondition, condition, { icopRefType: 'icopgrid' })
    };
    if (relyCondition && relyCondition.length) {
        var ems = relyCondition.split("=");
        var key = ems[0];
        busiParamJson['' + key] = ems[1];
    }
    var sysParamJson = { pagecs: Cookies.get("cookiets") };
    var appCode = getNccAppCode();
    if (appCode) {
        sysParamJson["appcode"] = appCode;
    }
    return {
        busiParamJson: JSON.stringify(busiParamJson),
        sysParamJson: sysParamJson
    };
}

/**
 * 针对ncc的列表型参照,对数据请求返回的结果集做转换
 * @param data
 * @author xg
 */
function convertNccGridRefResult() {
    var data = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    var pageNumber = parseInt(data.pageNumber);
    var pageSize = parseInt(data.pageSize);
    var pageCount = parseInt(data.pageCount);
    return {
        pageNumber: pageNumber,
        pageSize: pageSize,
        pageCount: pageCount,
        count: pageCount * pageSize, // ps：不是我脑残,后端返回的数据无总数,只能这样计算满足分页要求
        content: data.content || []
    };
}

/**
 * 针对ncc的树型参照,对数据请求的参数做转换
 * @param params 参照组件传递的参数
 * @param treeCondition 过滤条件
 * @author xg
 */
function convertNccTreeRefParams(params, treeCondition) {
    var _ref3 = params || {},
        condition = _ref3.condition;

    if (!condition) {
        condition = {};
    } else if (!_.isPlainObject(condition)) {
        condition = JSON.parse(condition);
    }
    if (!treeCondition) {
        treeCondition = {};
    } else if (!_.isPlainObject(treeCondition)) {
        treeCondition = JSON.parse(treeCondition);
    }
    var busiParamJson = {
        pageInfo: { pageSize: 50, pageIndex: -1 },
        pid: "",
        keyword: "",
        defineItems: [],
        queryCondition: _.assign({}, treeCondition, condition, { icopRefType: 'icoptree' })
    };
    var sysParamJson = { pagecs: Cookies.get("cookiets") };
    var appCode = getNccAppCode();
    if (appCode) {
        sysParamJson["appcode"] = appCode;
    }
    return {
        busiParamJson: JSON.stringify(busiParamJson),
        sysParamJson: sysParamJson
    };
}

/**
 * 获取ncc当前使用参照的页面appcode
 * @author xg
 */
function getNccAppCode() {
    var nccCache = DataUtil.localSessionReadObject("NCCAPPURL");
    if (nccCache && !_.isEmpty(nccCache)) {
        var hash = window.location.hash;
        if (hash) {
            var pageStr = hash.substr(hash.lastIndexOf("?") + 1, hash.length);
            if (pageStr && pageStr.length) {
                var ems = pageStr.split("=");
                if (ems && ems.length === 2) {
                    var pageId = ems[1];
                    if (nccCache.hasOwnProperty('' + pageId) && nccCache['' + pageId]) {
                        var param = decodeURIComponent(decodeURIComponent(nccCache['' + pageId]));
                        param = querystring.parse(param);
                        return param["c"];
                    }
                }
            }
        }
    }
    return null;
}

function doReload() {
    return function (dispatch) {
        return dispatch({
            type: _ReferConstants.DO_RELOAD_REF
        });
    };
}

function clearPreDatas() {
    return function (dispatch) {
        return dispatch({
            type: _ReferConstants.DO_CLEAR_PREDATAS
        });
    };
}

// // 查询常用
// export function getOften(serverUrl, params){
//     return function (dispatch) {
//         ajax.request({
//             method : 'GET',
//             url : ajax.fillUrlParams(serverUrl+GET_REFER_OFTEN, params),
//             type : 'json',
//             success : function(data) {
//                 if(data){
//                     return dispatch(getOftenSuccess(data.backData));
//                 }else{
//                     return dispatch(getOftenError(data.backMsg));
//                 }
//             },
//             error : function(data) {
//                 return dispatch(getOftenError(REFER_ERROR_MSG));
//             }
//         })
//     };
// }
//
// // 添加常用
// export function addOften(serverUrl, sendDatas, ifShowMsg){
//     return function (dispatch) {
//         ajax.request({
//             method : 'post',
//             url : serverUrl+SAVE_REFER_OFTEN,
//             send: JSON.stringify(sendDatas),
//             type : 'json',
//             success : function(data) {
//                 if(data && data.success === true){
//                     return dispatch(saveOftenSuccess(data.backData, ifShowMsg));
//                 }else{
//                     return dispatch(saveOftenError(data.backMsg));
//                 }
//             },
//             error : function(data) {
//                 return dispatch(saveOftenError(REFER_ERROR_MSG));
//             }
//         })
//     };
// }
//
// // 删除常用
// export function deleteOften(serverUrl, params){
//     return function (dispatch) {
//         ajax.request({
//             method : 'get',
//             url : ajax.fillUrlParams(serverUrl+DELETE_REFER_OFTEN, params),
//             type : 'json',
//             success : function(data) {
//                 if(data && data.success === true){
//                     return dispatch(deleteOftenSuccess(data.backData));
//                 }else{
//                     return dispatch(deleteOftenError(data.backMsg));
//                 }
//             },
//             error : function(data) {
//                 return dispatch(deleteOftenError(REFER_ERROR_MSG));
//             }
//         })
//     };
// }
// 保存参照内嵌行筛选条件
function saveInnerFilterObject(data) {
    return {
        type: _ReferConstants.SAVE_INNER_FILTER_VALUE,
        data: data
    };
}

// 清除参照内嵌行筛选条件
function delInnerFilterObject(refCode) {
    return {
        type: _ReferConstants.DEL_INNER_FILTER_VALUE,
        key: refCode
    };
}